// -------------------------------------------------------------------------------- 
//!  @file  mcp49x1.c                                                               
//!  MCP4901/MCP4911/MCP4921 ドライバソースファイル                                 
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "hardware/spi.h"
#include "config.h"
#include "mcp49x1.h"


#ifdef  MCP49x1_CFG_PIN_CS

// -------------------------------------------------------------------------------- 
//!  初期化                                                                         
//!                                                                                 
//!  @return  なし                                                                  
// -------------------------------------------------------------------------------- 
void mcp49x1_init( void )
{
	gpio_init( MCP49x1_CFG_PIN_CS ) ;
	gpio_put(MCP49x1_CFG_PIN_CS, 1) ;
	gpio_set_dir(MCP49x1_CFG_PIN_CS, GPIO_OUT) ;

	gpio_set_function(MCP49x1_CFG_PIN_SCK, GPIO_FUNC_SPI) ;
	gpio_set_function(MCP49x1_CFG_PIN_SDI, GPIO_FUNC_SPI) ;

	spi_init(MCP49x1_CFG_SPI, MCP49x1_CFG_SPI_CLK) ;
	spi_set_format(MCP49x1_CFG_SPI, 16, SPI_CPOL_1, SPI_CPHA_1, SPI_MSB_FIRST) ;
}


// -------------------------------------------------------------------------------- 
//!  書き込み                                                                       
//!                                                                                 
//!  @param [in] value  値                                                          
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
void RAMFUNC mcp49x1_write( uint16_t value )
{
	gpio_put(MCP49x1_CFG_PIN_CS, 0) ;

	spi_get_hw( MCP49x1_CFG_SPI )->dr = value | MCP49x1_CFG_COMMAND ;
}


// -------------------------------------------------------------------------------- 
//!  書き込み完了待ち                                                               
//!                                                                                 
//!  @return  なし                                                                  
// -------------------------------------------------------------------------------- 
void RAMFUNC mcp49x1_wait( void )
{
	while( spi_is_readable( MCP49x1_CFG_SPI ) ) {
		spi_get_hw( MCP49x1_CFG_SPI )->dr ;
	}

	while( spi_get_hw( MCP49x1_CFG_SPI )->sr & SPI_SSPSR_BSY_BITS ) {
		;
	}

	while( spi_is_readable( MCP49x1_CFG_SPI ) ) {
		spi_get_hw(MCP49x1_CFG_SPI)->dr ;
	}

    spi_get_hw( MCP49x1_CFG_SPI )->icr = SPI_SSPICR_RORIC_BITS ;

	gpio_put(MCP49x1_CFG_PIN_CS, 1) ;
}

#endif//MCP49x1_CFG_PIN_CS
