// -------------------------------------------------------------------------------- 
//!  @file  classic.c                                                               
//!  クラシックコントローラソースファイル                                           
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "config.h"
#include "pad.h"
#include "sfc.h"


#ifdef  PAD_CLASSIC_CFG_PIN_DETECT

//#define DBGTRACE(fmt, ...)			printf(fmt, __VA_ARGS__)
#define DBGTRACE(fmt, ...)			//


	enum {
		WAIT_COUNT					=	60,
		RETRY_COUNT					=	5
	} ;


static uint8_t						m_init ;						//!<
static uint8_t						m_wait ;						//!<
static uint8_t						m_retry ;						//!<
static uint8_t						m_type ;						//!<


// -------------------------------------------------------------------------------- 
//!  初期化                                                                         
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
void pad_classic_init( void )
{
	gpio_init( PAD_CLASSIC_CFG_PIN_DETECT ) ;
	gpio_set_function(PAD_CLASSIC_CFG_PIN_SDA, GPIO_FUNC_I2C) ;
	gpio_set_function(PAD_CLASSIC_CFG_PIN_SCL, GPIO_FUNC_I2C) ;

	i2c_init(PAD_CLASSIC_CFG_I2C, PAD_CLASSIC_CFG_I2C_SPEED) ;

	gpio_pull_down( PAD_CLASSIC_CFG_PIN_DETECT ) ;
//	gpio_pull_up( PAD_CLASSIC_CFG_PIN_SDA ) ;
//	gpio_pull_up( PAD_CLASSIC_CFG_PIN_SCL ) ;
	gpio_disable_pulls( PAD_CLASSIC_CFG_PIN_SDA ) ;
	gpio_disable_pulls( PAD_CLASSIC_CFG_PIN_SCL ) ;

	hw_write_masked(&padsbank0_hw->io[PAD_CLASSIC_CFG_PIN_SDA], PADS_BANK0_GPIO0_DRIVE_VALUE_12MA<<PADS_BANK0_GPIO0_DRIVE_LSB, PADS_BANK0_GPIO0_DRIVE_BITS) ;
	hw_write_masked(&padsbank0_hw->io[PAD_CLASSIC_CFG_PIN_SCL], PADS_BANK0_GPIO0_DRIVE_VALUE_12MA<<PADS_BANK0_GPIO0_DRIVE_LSB, PADS_BANK0_GPIO0_DRIVE_BITS) ;
}


// -------------------------------------------------------------------------------- 
//!  パッド状態を取得                                                               
//!                                                                                 
//!  @param [in,out] p_padInfo  パッド情報                                          
//!  @return                    なし                                                
// -------------------------------------------------------------------------------- 
void RAMFUNC pad_classic_get( PADINFO *p_padInfo )
{
	uint8_t		buff[21] ;
	uint32_t	status ;
	int			index, result ;


	if( !gpio_get( PAD_CLASSIC_CFG_PIN_DETECT ) ) {

		if( p_padInfo->type != PAD_TYPE_UNKNOWN ) {
			p_padInfo->type      = PAD_TYPE_UNKNOWN ;
			p_padInfo->status    = 0x0000 ;
			p_padInfo->analog[0] = 0x80 ;
			p_padInfo->analog[1] = 0x80 ;
			p_padInfo->analog[2] = 0x80 ;
			p_padInfo->analog[3] = 0x80 ;
		}

		m_init  = 0 ;
		m_wait  = WAIT_COUNT ;
		m_retry = 0 ;

	} else {

		if( m_wait ) {
			m_wait-- ;
			return ;
		}

		switch( m_init ) {
			case 0:
			{
				buff[0] = 0xf0 ;
				buff[1] = 0x55 ;
				result  = i2c_write_timeout_per_char_us(PAD_CLASSIC_CFG_I2C, 0x52, &buff[0], 2, false, 100) ;
				if( result != 2 ) {
					DBGTRACE("**** %s(%d) : %d\n", __func__, __LINE__, result) ;
					m_wait = WAIT_COUNT ;
					break ;
				}

				buff[0] = 0xfb ;
				buff[1] = 0x00 ;
				result  = i2c_write_timeout_per_char_us(PAD_CLASSIC_CFG_I2C, 0x52, &buff[0], 2, false, 100) ;
				if( result != 2 ) {
					DBGTRACE("**** %s(%d) : %d\n", __func__, __LINE__, result) ;
					m_wait = WAIT_COUNT ;
					break ;
				}

				sleep_us( 10 ) ;

				buff[0] = 0xfe ;
				buff[1] = 0x03 ;
				result  = i2c_write_timeout_per_char_us(PAD_CLASSIC_CFG_I2C, 0x52, &buff[0], 2, false, 100) ;
				if( result != 2 ) {
					DBGTRACE("**** %s(%d) : %d\n", __func__, __LINE__, result) ;
					m_wait = WAIT_COUNT ;
					break ;
				}
				m_init = 1 ;
				break ;
			}

			case 1:
			{
				buff[0] = 0xfa ;
				result  = i2c_write_timeout_per_char_us(PAD_CLASSIC_CFG_I2C, 0x52, &buff[0], 1, false, 100) ;
				if( result != 1 ) {
					DBGTRACE("**** %s(%d) : %d\n", __func__, __LINE__, result) ;
					m_wait = WAIT_COUNT ;
					break ;
				}
				m_init = 2 ;
				break ;
			}

			case 2:
			{
				result = i2c_read_timeout_per_char_us(PAD_CLASSIC_CFG_I2C, 0x52, &buff[0], 6, false, 200) ;
				if( result != 6 ) {
					DBGTRACE("**** %s(%d) : %d\n", __func__, __LINE__, result) ;
					m_init = 0 ;
					m_wait = WAIT_COUNT ;
					break ;
				}

				// デバイス ID が異なるならやり直し
				if( (buff[4]!=0x03) || (buff[5]!=0x01) ) {
					DBGTRACE("**** %s(%d) : %d\n", __func__, __LINE__, result) ;

					m_wait = WAIT_COUNT ;

					if( ++m_retry == RETRY_COUNT ) {
						m_init  = 0 ;
						m_retry = 0 ;
						break ;
					}

					m_init = 1 ;
					break ;
				}

				m_type = PAD_TYPE_CLASSIC ;
				if( buff[0] ) {
					m_type = PAD_TYPE_MINISFC ;
				}

				buff[0] = 0x00 ;
				result  = i2c_write_timeout_per_char_us(PAD_CLASSIC_CFG_I2C, 0x52, &buff[0], 1, false, 100) ;
				if( result != 1 ) {
					DBGTRACE("**** %s(%d) : %d\n", __func__, __LINE__, result) ;
					m_init = 0 ;
					m_wait  = WAIT_COUNT ;
					break ;
				}
				m_init = 3 ;
				break ;
			}

//			case 3:
			default:
			{
				result = i2c_read_timeout_per_char_us(PAD_CLASSIC_CFG_I2C, 0x52, &buff[0], 21, false, 100) ;
				if( result != 21 ) {
					DBGTRACE("**** %s(%d) : %d\n", __func__, __LINE__, result) ;
					m_init = 0 ;
					m_wait = WAIT_COUNT ;
					break ;
				}

				// データ異常確認
				for(index=8; index<21; index++) {
					if( buff[index] ) {
						DBGTRACE("**** %s(%d) : %d, %d\n", __func__, __LINE__, index, result) ;
						break ;
					}
				}

				p_padInfo->type      = m_type ;
				p_padInfo->status    = 0x0000 ;
				p_padInfo->analog[0] = 0x80 ;
				p_padInfo->analog[1] = 0x80 ;
				p_padInfo->analog[2] = 0x80 ;
				p_padInfo->analog[3] = 0x80 ;

				if( index == 21 ) {

					// [0] = LX
					// [1] = RX
					// [2] = LY
					// [3] = RY
					// [4] = Ｌ
					// [5] = Ｒ
					// [6] = 右 | 下 | Ｌ | セ | Ｈ | ス | Ｒ | １
					// [7] = ZL | ｂ | ｙ | ａ | ｘ | ZR | 左 | 上

					status            = ~(((uint32_t )buff[6] << 8) | buff[7]) ;
					p_padInfo->status = (((status >> 1) & 1) <<  PAD_LEFT_)   |
						                (((status >>15) & 1) <<  PAD_RIGHT_)  |
						                (((status >> 0) & 1) <<  PAD_UP_)     |
						                (((status >>14) & 1) <<  PAD_DOWN_)   |
						                (((status >>12) & 1) <<  PAD_SELECT_) |
						                (((status >>10) & 1) <<  PAD_START_)  |
						                (((status >> 6) & 1) << (PAD_BTN_+0)) |
						                (((status >> 4) & 1) << (PAD_BTN_+1)) |
						                (((status >> 5) & 1) << (PAD_BTN_+2)) |
						                (((status >> 3) & 1) << (PAD_BTN_+3)) |
						                (((status >>13) & 1) <<  PAD_LB_)     |
						                (((status >> 9) & 1) <<  PAD_RB_) ;

					// クラシックコントローラ
					if( m_type == PAD_TYPE_CLASSIC ) {
						p_padInfo->status   |= (((status >>11) & 1) << (PAD_BTN_+4)) |
						                       (((status >> 7) & 1) << (PAD_BTN_+5)) |
						                       (((status >> 2) & 1) << (PAD_BTN_+6)) ;
						p_padInfo->analog[0] =  buff[0] ;
						p_padInfo->analog[1] = -buff[2] ;
						p_padInfo->analog[2] =  buff[1] ;
						p_padInfo->analog[3] = -buff[3] ;
					}

				}

				sleep_us( 30 ) ;

				buff[0] = 0x00 ;
				result  = i2c_write_timeout_per_char_us(PAD_CLASSIC_CFG_I2C, 0x52, &buff[0], 1, false, 100) ;
				if( result != 1 ) {
					DBGTRACE("**** %s(%d) : %d\n", __func__, __LINE__, result) ;
					m_init = 0 ;
					m_wait = WAIT_COUNT ;
					break ;
				}
				return ;
			}
		}
	}

	p_padInfo->type      = PAD_TYPE_UNKNOWN ;
	p_padInfo->status    = 0x0000 ;
	p_padInfo->analog[0] = 0x80 ;
	p_padInfo->analog[1] = 0x80 ;
	p_padInfo->analog[2] = 0x80 ;
	p_padInfo->analog[3] = 0x80 ;
}

#endif//PAD_CLASSIC_CFG_PIN_DETECT
