// -------------------------------------------------------------------------------- 
//!  @file  pad.h                                                                   
//!  設定ファイル                                                                   
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __PAD__PAD_H__INCLUDED__
#define __PAD__PAD_H__INCLUDED__

	//! パッド ID
	enum {
		PAD_ID_SERIAL,												//!< シリアル通信
		PAD_ID_PCE,													//!< PC Engine
		PAD_ID_MD,													//!< MEGA DRIVE
		PAD_ID_SFC,													//!< SUPER Famicom
		PAD_ID_PS,													//!< PlayStation
		PAD_ID_CLASSIC,												//!< クラシックコントローラ
		PAD_ID_UNUSED,												//!< 未使用
		PAD_ID_MAX
	} ;

	//! パッド種類
	enum {
		PAD_TYPE_UNKNOWN,											//!< 不明
		PAD_TYPE_SERIAL,											//!< シリアル通信
		PAD_TYPE_PCE,												//!< PC エンジン
		PAD_TYPE_PCE_6B,											//!< PC エンジン（６ボタン）
		PAD_TYPE_MD,												//!< メガドライブ
		PAD_TYPE_MD_6B,												//!< メガドライブ（６ボタン）
		PAD_TYPE_PS,												//!< PlayStation
		PAD_TYPE_PS_ANALOG,											//!< PlayStation（アナログ）
		PAD_TYPE_CLASSIC,											//!< クラシックコントローラ
		PAD_TYPE_MINISFC,											//!< ニンテンドークラシックミニ スーパーファミコン
		PAD_TYPE_MAX
	} ;

	//! パッド状態
	enum {
		PAD_BTN_					=	0,
		PAD_LB_						=	8,
		PAD_RB_						=	9,
		PAD_SELECT_					=	10,
		PAD_START_					=	11,
		PAD_DIR_					=	12,
		PAD_LEFT_					=	PAD_DIR_+0,
		PAD_RIGHT_					=	PAD_DIR_+1,
		PAD_UP_						=	PAD_DIR_+2,
		PAD_DOWN_					=	PAD_DIR_+3,

		PAD_LB						=	1 << PAD_LB_,
		PAD_RB						=	1 << PAD_RB_,
		PAD_SELECT					=	1 << PAD_SELECT_,
		PAD_START					=	1 << PAD_START_,

		PAD_LEFT					=	1 << PAD_LEFT_,
		PAD_RIGHT					=	1 << PAD_RIGHT_,
		PAD_UP						=	1 << PAD_UP_,
		PAD_DOWN					=	1 << PAD_DOWN_
	} ;

#define PAD_BTN( n )				(1 << (PAD_BTN_+(n)))			//!< ボタン状態ビット

	//! パッド情報
	typedef struct {
		uint8_t						type ;							//!< パッド種類
		uint8_t						reserved ;						//!< 予約（未使用）
		uint16_t					status ;						//!< 状態
		uint8_t						analog[4] ;						//!< アナログ
	} PADINFO ;


extern PADINFO						g_padInfo ;						//!< パッド情報


extern uint8_t         pad_init( void ) ;
extern void            pad_start( void ) ;
extern void    RAMFUNC pad_update( void ) ;
extern uint8_t RAMFUNC pad_getId( void ) ;


#endif//__PAD__PAD_H__INCLUDED__
