// -------------------------------------------------------------------------------- 
//!  @file  ay-3-8910.h                                                             
//!  AY-3-8910 音源 PCM データジェネレータヘッダファイル                            
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __PCM__AY_3_8910_H__INCLUDED__
#define __PCM__AY_3_8910_H__INCLUDED__

// レジスタ
//     ch.A/B/C       |----N--T|---MLLLL|----FFFF|FFFFFFFF|
//     noise/envelope |---PPPPP|----CATH|FFFFFFFF|FFFFFFFF|
//
//     N  Noise
//     T  Tone
//     M  amplitude Mode
//     L  4-bit "fixed" amplitude Level
//     F  Fine Tune
//     P  Period Control
//     C  Continue
//     A  Alternate
//     T  Attach
//     H  Hold


	enum {
		PCMAY38910_CHANNEL_COUNT	=	3							//!< チャンネル数
	} ;

	//!< レジスタ
	typedef union {
		uint8_t						n8[4] ;							//!<  8bit
		uint16_t					n16[2] ;						//!< 16bit
		uint32_t					n32 ;							//!< 32bit
	} PCMAY38910REG ;

	//! チャンネル情報
	typedef struct {
		PCMAY38910REG				reg ;							//!< レジスタ値
		uint32_t					freqCount ;						//!< 周波数カウンタ
		uint8_t						volume ;						//!< 音量
		uint8_t						output ;						//!< 出力
	} PCMAY38910CHANNELINFO ;

	//! AY-3-8910 情報
	typedef struct {
		PCMAY38910CHANNELINFO		channelInfo[PCMAY38910_CHANNEL_COUNT] ;
																	//!< チャンネル情報

		PCMAY38910REG				reg ;							//!< ノイズ・エンベロープレジスタ値

		//! ノイズ
		struct {
			uint32_t				freqCount ;						//!< 周波数カウンタ
			uint32_t				output ;						//!< 出力状態
		} noise ;

		//! エンベロープ
		struct {
			uint8_t					reg ;							//!< コントロールレジスタ
			uint8_t					ctrl ;							//!< コントロール状態
			uint8_t					update ;						//!< 更新フラグ
			uint8_t					volume ;						//!< 音量
			uint32_t				freqCount ;						//!< 周波数カウンタ
		} envelope ;
	} PCMAY38910INFO ;


#define pcm_ay_3_8910_setChannelReg(p, ch, value)		((p)->channelInfo[ch].reg.n32 = (value))
#define pcm_ay_3_8910_setEnvelopeShape(p, value)		{ (p)->reg.n8[2]  = (value) ; (p)->envelope.update = 1 ; }
#define pcm_ay_3_8910_setEnvelopePeriod(p, value)		{ (p)->reg.n16[0] = (value) ;                            }
#define pcm_ay_3_8910_setNoiseEnvelopeReg(p, value)		{ (p)->reg.n32    = (value) ; (p)->envelope.update = 1 ; }


extern void            pcm_ay_3_8910_reset( PCMAY38910INFO *p ) ;
extern int16_t RAMFUNC pcm_ay_3_8910_update( PCMAY38910INFO *p ) ;


#endif//__PCM__AY_3_8910_H__INCLUDED__
