// -------------------------------------------------------------------------------- 
//!  @file  dpcm3.c                                                                 
//!  3bit DPCM ジェネレータソースファイル                                           
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "config.h"
#include "dpcm3.h"


#ifdef  DPCM3_CFG_CHANNEL_COUNT

// -------------------------------------------------------------------------------- 
//!  リセット                                                                       
//!                                                                                 
//!  @param [out] p  3bit DPCM 情報                                                 
//!  @return         なし                                                           
// -------------------------------------------------------------------------------- 
void dpcm3_reset( DPCM3INFO *p )
{
	memset(p, 0, sizeof( DPCM3INFO )) ;
}


// -------------------------------------------------------------------------------- 
//!  更新                                                                           
//!                                                                                 
//!  @param [in,out] p  3bit DPCM 情報                                              
//!  @return            出力データ                                                  
// -------------------------------------------------------------------------------- 
int16_t RAMFUNC dpcm3_update( DPCM3INFO *p )
{
	DPCM3CHANNELINFO	*p_channelInfo, *p_channelInfoEnd ;
	uint16_t			const *p_dpcmData ;
	int32_t				wave, bitData ;


	p_channelInfo    = &p->channelInfo[0] ;
	p_channelInfoEnd = &p->channelInfo[DPCM3_CFG_CHANNEL_COUNT] ;
	wave             = 0 ;

	do {
		// 設定
		p_dpcmData = p_channelInfo->p_setDpcmData ;
		if( p_dpcmData != NULL ) {
			p_channelInfo->p_setDpcmData = NULL ;
			p_channelInfo->size          = p_dpcmData[0] ;
			p_channelInfo->initData[0]   = p_dpcmData[1] ;
			p_channelInfo->initData[1]   = p_dpcmData[2] ;
			p_channelInfo->initData[2]   = p_dpcmData[3] ;
			p_channelInfo->initData[3]   = p_dpcmData[4] ;
			p_channelInfo->dpcmData      = p_dpcmData[5] ;
			p_channelInfo->count         = 16 ;
			p_channelInfo->value         = 0 ;
			p_channelInfo->p_dpcmData    = &p_dpcmData[6] ;
		}

		// 更新
		if( p_channelInfo->p_dpcmData != NULL ) {
			if( p_channelInfo->count < 3 ) {
				p_channelInfo->dpcmData |= (uint32_t )*p_channelInfo->p_dpcmData++ << p_channelInfo->count ;
				p_channelInfo->count    += 16 ;
			}

			bitData                   = p_channelInfo->dpcmData & 7 ;
			p_channelInfo->dpcmData >>= 3 ;
			p_channelInfo->count     -= 3 ;

			p_channelInfo->value += p_channelInfo->deltaData[bitData] ;
			wave                 += p_channelInfo->value ;

			// 終了
			if( !--p_channelInfo->size ) {
				p_channelInfo->p_dpcmData = NULL ;
				p_channelInfo->value      = 0 ;
			}
		}
	} while( ++p_channelInfo != p_channelInfoEnd ) ;

	return (wave<<DPCM3_CFG_ADJUSTVOLUME) ;
}

#endif//DPCM3_CFG_CHANNEL_COUNT
