// -------------------------------------------------------------------------------- 
//!  @file  dpcm3.h                                                                 
//!  3bit DPCM ジェネレータヘッダファイル                                           
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __PCM__DPCM3_H__INCLUDED__
#define __PCM__DPCM3_H__INCLUDED__


#ifdef  DPCM3_CFG_CHANNEL_COUNT

	//! 3bit DPCM チャンネル情報
	typedef struct {
		uint16_t					const *p_setDpcmData ;			//!< DPCM 設定データ
		uint16_t					const *p_dpcmData ;				//!< DPCM データ
		union {
			uint16_t				initData[4] ;					//!< デルタデータアクセス用
			int8_t					deltaData[8] ;					//!< デルタデータ
		} ;
		uint32_t					dpcmData ;						//!< カレントデルタデータ
		uint16_t					size ;							//!< デルタデータ残りサイズ
		uint8_t						count ;							//!< デルタデータ残りビット数
		int8_t						value ;							//!< カレントデータ値
	} DPCM3CHANNELINFO ;

	//! 3bit DPCM 情報
	typedef struct {
		DPCM3CHANNELINFO			channelInfo[DPCM3_CFG_CHANNEL_COUNT] ;
																	//!< チャンネル情報
	} DPCM3INFO ;


extern void            dpcm3_reset( DPCM3INFO *p ) ;
extern int16_t RAMFUNC dpcm3_update( DPCM3INFO *p ) ;


// -------------------------------------------------------------------------------- 
//!  データ設定                                                                     
//!                                                                                 
//!  @param [in,out] p           3bit DPCM 情報                                     
//!  @param [in]     channel     チャンネル                                         
//!  @param [in]     p_dpcmData  3bit DPCM データ                                   
//!  @return                     なし                                               
// -------------------------------------------------------------------------------- 
static inline void dpcm3_setData(DPCM3INFO *p, uint8_t channel, uint16_t const *p_dpcmData)
{
	p->channelInfo[channel].p_setDpcmData = p_dpcmData ;
}


#endif//DPCM3_CFG_CHANNEL_COUNT


#endif//__PCM__DPCM3_H__INCLUDED__
