// -------------------------------------------------------------------------------- 
//!  @file  namcot163.c                                                             
//!  NAMCOT-163 音源 PCM データジェネレータソースファイル                           
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "config.h"
#include "namcot163.h"


#ifdef  PCMNAMCOT163_CFG_BASECLK

	enum {
		CLK							=	(PCMNAMCOT163_CFG_BASECLK * PCMNAMCOT163_CFG_TIME + (1000000 / 2)) / 1000000
																	//!< クロック
	} ;


// -------------------------------------------------------------------------------- 
//!  リセット                                                                       
//!                                                                                 
//!  @param [out] p  NAMCOT-163 情報                                                
//!  @return         なし                                                           
// -------------------------------------------------------------------------------- 
void pcm_namcot163_reset( PCMNAMCOT163INFO *p )
{
	memset(p, 0, sizeof( PCMNAMCOT163INFO )) ;
	p->channelMax = 1 ;
}


// -------------------------------------------------------------------------------- 
//!  更新                                                                           
//!                                                                                 
//!  @param [in,out] p  NAMCOT-163 情報                                             
//!  @return            出力データ                                                  
// -------------------------------------------------------------------------------- 
int16_t RAMFUNC pcm_namcot163_update( PCMNAMCOT163INFO *p )
{
	PCMNAMCOT163CHANNELINFO		*p_channelInfo, *p_channelInfoEnd ;
	int32_t						output ;
	uint32_t					value, freq, max, volume, address ;
	int							index ;


	if( !p->enable ) {
		return 0 ;
	}

	// 更新
	for(index=0; index<CLK; index++) {
		if( p->channelIndex >= p->channelMax ) {
			p->channelIndex  = 0 ;
		}

		p_channelInfo = &p->channelInfo[7-p->channelIndex] ;
		p->channelIndex++ ;

		freq = (p_channelInfo->reg[0] << 14) >> 14 ;
		if( freq ) {
			freq += p_channelInfo->reg[2] ;

			max = (uint32_t )(0x40 - (p_channelInfo->regb[5] & 0x3f)) << 18 ;
			if( freq >= max ) {
				freq -= max ;
			}

			p_channelInfo->reg[2] = freq ;
		}
	}

	//
	p_channelInfo    = &p->channelInfo[8-p->channelMax] ;
	p_channelInfoEnd = &p->channelInfo[8] ;
	output           = 0 ;

	do {
		value = p_channelInfo->reg[0] ;

		freq = (value << 14) >> 14 ;
		if( freq ) {
			volume = ((value << 4) >> 28) + 1 ;

			address = p_channelInfo->regb[4] + p_channelInfo->regb[10] ;
			value   = p->wave[address/2] ;
			if( address & 1 ) {
				value >>= 4 ;
			}

			output += (((int32_t )(value & 15) - 8) * volume) ;
		}
	} while( ++p_channelInfo != p_channelInfoEnd ) ;

	return (int16_t )output ;
}
#endif//PCMNAMCOT163_CFG_BASECLK
