// -------------------------------------------------------------------------------- 
//!  @file  scc.c                                                                   
//!  SCC 音源 PCM データジェネレータソースファイル                                  
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "config.h"
#include "scc.h"


#ifdef  PCMSCC_CFG_BASECLK

	enum {
		CLK							=	(PCMSCC_CFG_BASECLK * PCMSCC_CFG_TIME + (1000000 / 2)) / 1000000
																	//!< クロック
	} ;


// -------------------------------------------------------------------------------- 
//!  リセット                                                                       
//!                                                                                 
//!  @param [out] p  SCC 情報                                                       
//!  @return         なし                                                           
// -------------------------------------------------------------------------------- 
void pcm_scc_reset( PCMSCCINFO *p )
{
	static const uint8_t	offset[PCMSCC_CHANNEL_COUNT]={
								0 * PCMSCC_CHANNEL_WAVE_SIZE,
								1 * PCMSCC_CHANNEL_WAVE_SIZE,
								2 * PCMSCC_CHANNEL_WAVE_SIZE,
								3 * PCMSCC_CHANNEL_WAVE_SIZE,
								3 * PCMSCC_CHANNEL_WAVE_SIZE
							} ;
	int						index ;


	memset(p, 0, sizeof( PCMSCCINFO )) ;

	for(index=0; index<PCMSCC_CHANNEL_COUNT; index++) {
		p->channelInfo[index].offset = offset[index] ;
	}
}


// -------------------------------------------------------------------------------- 
//!  更新                                                                           
//!                                                                                 
//!  @param [in,out] p  SCC 情報                                                    
//!  @return            出力データ                                                  
// -------------------------------------------------------------------------------- 
int16_t RAMFUNC pcm_scc_update( PCMSCCINFO *p )
{
	PCMSCCCHANNELINFO	*p_channelInfo ;
	uint8_t				enable ;
	int32_t				output ;
	uint32_t			count, volume, freq ;


	p_channelInfo = &p->channelInfo[0] ;
	enable        = p->enable ;
	output        = 0 ;


	while( enable ) {
		if( enable & 1 ) {
			volume = p_channelInfo->reg >> 12 ;
			freq   = p_channelInfo->reg & 0x0fff ;

			if( freq >= 8 ) {
				count = p_channelInfo->count + CLK ;
				while( count >= freq ) {
					count               -= freq ;
					p_channelInfo->index = (p_channelInfo->index + 1) & PCMSCC_CHANNEL_WAVE_MASK ;
				}
				p_channelInfo->count = (uint16_t )count ;

				output += ((int32_t )(volume * p->wave[p_channelInfo->offset+p_channelInfo->index]) / 8) ;
			}
		}

		p_channelInfo++ ;
		enable >>= 1 ;
	}

	return (int16_t )output ;
}

#endif//PCMSCC_CFG_BASECLK
