// -------------------------------------------------------------------------------- 
//!  @file  sn76489.c                                                               
//!  SN76489 音源 PCM データジェネレータソースファイル                              
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "config.h"
#include "sn76489.h"


#ifdef  PCMSN76489_CFG_BASECLK

	enum {
		CLK							=	(PCMSN76489_CFG_BASECLK * (PCMSN76489_CFG_TIME * 2) / 32 + (1000000 / 2)) / 1000000
																	//!< クロック
	} ;

static uint16_t						m_volume[16]={					//!< 音量
										8192/32-1,
										6507/32-1,
										5169/32-1,
										4106/32-1,
										3261/32-1,
										2591/32-1,
										2058/32-1,
										1635/32-1,
										1299/32-1,
										1032/32-1,
										 820/32-1,
										 651/32-1,
										 517/32-1,
										 411/32-1,
										 327/32-1,
										      0
									} ;


// -------------------------------------------------------------------------------- 
//!  リセット                                                                       
//!                                                                                 
//!  @param [out] p  SN76489 情報                                                   
//!  @return         なし                                                           
// -------------------------------------------------------------------------------- 
void pcm_sn76489_reset( PCMSN76489INFO *p )
{
	memset(p, 0, sizeof( PCMSN76489INFO )) ;
	p->channelInfo[0].reg = 0xf000 ;
	p->channelInfo[1].reg = 0xf000 ;
	p->channelInfo[2].reg = 0xf000 ;
	p->channelInfo[3].reg = 0xf800 ;
}


// -------------------------------------------------------------------------------- 
//!  更新                                                                           
//!                                                                                 
//!  @param [in,out] p  SN76489 情報                                                
//!  @return            出力データ                                                  
// -------------------------------------------------------------------------------- 
int16_t RAMFUNC pcm_sn76489_update( PCMSN76489INFO *p )
{
	PCMSN76489CHANNELINFO	*p_channelInfo, *p_channelInfoEnd ;
	int32_t					output ;
	uint32_t				reg, freq, atte ;


	p_channelInfo    = &p->channelInfo[0] ;
	p_channelInfoEnd = &p->channelInfo[PCMSN76489_CHANNEL_COUNT] ;
	output           = 0 ;

	do {
		reg  = p_channelInfo->reg ;
		freq = reg & 0x03ff ;
		atte = reg >> 12 ;

		if( freq >= 16 ) {
			p_channelInfo->count += CLK ;
			while( p_channelInfo->count >= freq ) {
				p_channelInfo->count -= freq ;

				if( !(reg&(1<<11)) ) {	// トーン
					p_channelInfo->output ^= 1 ;
				} else
				if( reg & (1<<10) ) {	// ホワイトノイズ
					p_channelInfo->output = (p_channelInfo->output >> 1) | (((p_channelInfo->output << 12) ^ (p_channelInfo->output << 15)) & 0x8000) ;
				} else {				// 同期ノイズ
					p_channelInfo->output = (p_channelInfo->output >> 1) |                                   (p_channelInfo->output << 15) ;
				}
			}

			if( p_channelInfo->output & 1 ) {
				output += m_volume[atte] ;
			} else {
				output -= m_volume[atte] ;
			}
		}
	} while( ++p_channelInfo != p_channelInfoEnd ) ;

	return (int16_t )output ;
}


// -------------------------------------------------------------------------------- 
//!  レジスタ設定                                                                   
//!                                                                                 
//!  @param [in,out] p        SN76489 情報                                          
//!  @param [in]     channel  チャンネル番号                                        
//!  @param [in]     value    レジスタ値                                            
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
void RAMFUNC pcm_sn76489_setReg(PCMSN76489INFO *p, uint8_t channel, uint16_t value)
{
	PCMSN76489CHANNELINFO	*p_channelInfo ;
	uint16_t				update ;


	p_channelInfo      = &p->channelInfo[channel] ;
	update             = p_channelInfo->reg ^ value ;
	p_channelInfo->reg = value ;

	if( update & 0x0c00 ) {
		p_channelInfo->output = 0x8000 ;
	}
}

#endif//PCMSN76489_CFG_BASECLK
