// -------------------------------------------------------------------------------- 
//!  @file  sn76489.h                                                               
//!  SN76489 音源 PCM データジェネレータヘッダファイル                              
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __PCM__SN76489_H__INCLUDED__
#define __PCM__SN76489_H__INCLUDED__


// レジスタ |AAAA|NBFF|FFFF|FFFF|
//        A  アッテネーション
//        N  0:トーン,     1:ノイズ
//        B  0:同期ノイズ, 1:ホワイトノイズ
//        F  周波数


	enum {
		PCMSN76489_CHANNEL_COUNT	=	4							//!< チャンネル数
	} ;

	//! チャンネル情報
	typedef struct {
		uint16_t					reg ;							//!< レジスタ値
		uint16_t					count ;							//!< 周波数カウンタ
		uint16_t					output ;						//!< 出力＋ノイズ
	} PCMSN76489CHANNELINFO ;

	//! SN76489 情報
	typedef struct {
																	//!< チャンネル情報
		PCMSN76489CHANNELINFO		channelInfo[PCMSN76489_CHANNEL_COUNT] ;
	} PCMSN76489INFO ;


extern void            pcm_sn76489_reset( PCMSN76489INFO *p ) ;
extern int16_t RAMFUNC pcm_sn76489_update( PCMSN76489INFO *p ) ;
extern void    RAMFUNC pcm_sn76489_setReg(PCMSN76489INFO *p, uint8_t channel, uint16_t value) ;


// -------------------------------------------------------------------------------- 
//!  アッテネーション設定                                                           
//!                                                                                 
//!  @param [in,out] p        SN76489 情報                                          
//!  @param [in]     channel  チャンネル番号                                        
//!  @param [in]     atte     アッテネーション                                      
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static inline void pcm_sn76489_setAtte(PCMSN76489INFO *p, uint8_t channel, uint8_t atte)
{
	PCMSN76489CHANNELINFO	*p_channelInfo ;


	p_channelInfo      = &p->channelInfo[channel] ;
	p_channelInfo->reg = (p_channelInfo->reg & 0x0fff) | (atte << 12) ;
}


// -------------------------------------------------------------------------------- 
//!  周波数設定                                                                     
//!                                                                                 
//!  @param [in,out] p        SN76489 情報                                          
//!  @param [in]     channel  チャンネル番号                                        
//!  @param [in]     freq     周波数                                                
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static inline void pcm_sn76489_setFreq(PCMSN76489INFO *p, uint8_t channel, uint16_t freq)
{
	PCMSN76489CHANNELINFO	*p_channelInfo ;


	p_channelInfo      = &p->channelInfo[channel] ;
	p_channelInfo->reg = (p_channelInfo->reg & 0xfc00) | freq;
}


#endif//__PCM__SN76489_H__INCLUDED__
